package com.bohuikeji.frame.module.admin.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bohuikeji.frame.module.admin.base.ServiceImplBase;
import com.bohuikeji.frame.module.admin.component.RedisUtil;
import com.bohuikeji.frame.module.admin.entity.sys.SysConfig;
import com.bohuikeji.frame.module.admin.mapper.SysConfigMapper;
import com.bohuikeji.frame.module.admin.service.ISysConfigService;
import com.bohuikeji.frame.module.admin.utils.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * <p>
 * 系统配置项 服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-08-09
 */
@Slf4j
@Service
public class SysConfigServiceImpl extends ServiceImplBase<SysConfigMapper, SysConfig> implements ISysConfigService {
    @Autowired
    RedisUtil redisUtil;
    @Autowired
    ISysConfigService configService;

    /**
     * 获取当前配置环境
     */
    @Value("${spring.profiles.active}")
    private String active;

    @Override
    public SysConfig getOneByKey(String key) {
        SysConfig sysConfig=redisUtil.get("config_key_"+key,SysConfig.class);
        if(sysConfig==null){
            sysConfig=getOne(new QueryWrapper<SysConfig>().eq("config_key",key));
            //有效期5分钟
            redisUtil.set("config_key_"+key,sysConfig,60*5L);
        }
        return sysConfig;
    }

    @Override
    public List<SysConfig> getListByGroup(String groupKey) {
        List<SysConfig> configList=redisUtil.getList(groupKey,SysConfig.class);
        if (configList==null||configList.size()==0){
            configList=list(new QueryWrapper<SysConfig>().eq("config_group",groupKey));
            //有效期2小时
            redisUtil.set(groupKey,configList,3600*2L);
        }
        return configList;
    }

    @Override
    public JSONObject jsonObject(SysConfig sysConfig) {
        JSONObject jsonObject = new JSONObject(true);
        jsonObject.put("grade",sysConfig.getConfigName());
        jsonObject.put("totel",sysConfig.getConfigDataOne());
        jsonObject.put("value",sysConfig.getConfigDataTwo());
        jsonObject.put("section",sysConfig.getConfigValue());
        jsonObject.put("explain",sysConfig.getConfigMark());
        return jsonObject;
    }

    /**
     * 根据code获取访问令牌
     * 时间有效期内根据刷新code刷新新的访问令牌
     * @param code
     * @param res
     * @return
     */
    @Override
    public SysConfig getHuolalaToken(String code,JSONObject res) {
        SysConfig config = configService.getOneByKey(active);
        log.info("获取访问令牌:{}",res);
        if(StringUtils.isNotBlank(code)){
            if(res.getString("msg").equals("成功")){
                config.setConfigDataOne(res.getJSONObject("data").getString("access_token"));
                config.setConfigDataTwo(res.getJSONObject("data").getString("refresh_token"));
                config.setConfigGroup(res.getJSONObject("data").getString("auth_end_time"));
                config.setConfigDefault(res.getJSONObject("data").getString("auth_mobile"));
                configService.save(config);
            }
        }else {
            if(res.getString("msg").equals("成功")){
                config.setConfigDataOne(res.getJSONObject("data").getString("access_token"));
                config.setConfigDataTwo(res.getJSONObject("data").getString("refresh_token"));
                config.setConfigGroup(res.getJSONObject("data").getString("auth_end_time"));
                config.setConfigDefault(res.getJSONObject("data").getString("auth_mobile"));
                configService.updateById(config);
            }
        }
        return config;
    }
}
